/*
 * Name:    make_mask.c
 *
 * Purpose: Used to generate an area detector mask image by
 *          masking off all pixels in a sample image that are
 *          lower than a specified threshold.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2007 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mx_util.h"
#include "mx_stdint.h"
#include "mx_image.h"

int
main( int argc, char *argv[] )
{
	MX_IMAGE_FRAME *sample_frame, *mask_frame;
	uint16_t *uint16_sample_data, *uint16_mask_data;
	unsigned long i, num_pixels, threshold, num_masked_pixels;
	mx_status_type mx_status;

	if ( argc != 4 ) {
		fprintf(stderr,
		"Usage: make_mask threshold sample_filename mask_filename\n");
		exit(1);
	}

	sample_frame = NULL;
	mask_frame   = NULL;

	threshold = atol( argv[1] );

	/* Read the sample frame. */

	mx_status = mx_image_read_smv_file( &sample_frame, argv[2] );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	if ( MXIF_IMAGE_FORMAT(sample_frame) != MXT_IMAGE_FORMAT_GREY16 ) {
		fprintf(stderr,
		"Sample frame '%s' is not a 16-bit greyscale image.\n",
			argv[2]);
		exit(1);
	}

	uint16_sample_data = sample_frame->image_data;

	/* Create a new empty frame with the same configuration as
	 * the sample frame.
	 */

	mx_status = mx_image_alloc( &mask_frame,
					MXIF_ROW_FRAMESIZE(sample_frame),
					MXIF_COLUMN_FRAMESIZE(sample_frame),
					MXIF_IMAGE_FORMAT(sample_frame),
					MXIF_BYTE_ORDER(sample_frame),
					MXIF_BYTES_PER_PIXEL(sample_frame),
					sample_frame->header_length,
					sample_frame->image_length );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	uint16_mask_data = mask_frame->image_data;

	memset( uint16_mask_data, 0, sample_frame->image_length );

	num_pixels = sample_frame->image_length / 2L;

	num_masked_pixels = 0L;

	for ( i = 0; i < num_pixels; i++ ) {
		if ( uint16_sample_data[i] >= threshold ) {
			uint16_mask_data[i] = 1L;
		} else {
			uint16_mask_data[i] = 0L;

			num_masked_pixels++;
		}
	}

	mx_status = mx_image_write_smv_file( mask_frame, argv[3] );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	fprintf( stderr,
	"Mask frame '%s' successfully created with %lu masked pixels.\n",
		argv[3], num_masked_pixels );

	exit(0);
}

