/*
 * Name:    mx_offline.c
 *
 * Purpose: Performs offline corrections of MX area detector images.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2007 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_image.h"
#include "mx_area_detector.h"

int
main( int argc, char *argv[] )
{
	static const char fname[] = "mx_offline";

	MX_RECORD *record_list, *ad_record;
	char *ad_name;
	char *database_filename;
	char *original_filename;
	char *corrected_filename;
	long bytes_per_frame, format_type, row_framesize, col_framesize;
	char format_name[MXU_IMAGE_FORMAT_NAME_LENGTH+1];
	double bytes_per_pixel;
	unsigned long correction_flags;

	mx_status_type mx_status;

	if ( argc != 6 ) {
		fprintf(stderr,
			"Usage: mx_offline mx_database_file ad_name flags "
			"original_image corrected_image\n" );
		exit(1);
	}

	database_filename  = argv[1];
	ad_name            = argv[2];
	correction_flags   = mx_hex_string_to_unsigned_long( argv[3] );
	original_filename  = argv[4];
	corrected_filename = argv[5];

	/* Read an MX database file _without_ attempting to connect
	 * to the associated hardware devices.  In other words, do not
	 * invoke mx_initialize_hardware().
	 */

	mx_status = mx_initialize_runtime();

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_initialize_drivers();

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	record_list = mx_initialize_record_list();

	if ( record_list == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, fname,
				"Unable to setup an MX record list." );
		exit(mx_status.code);
	}

	mx_status = mx_read_database_file( record_list, database_filename, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_finish_database_initialization( record_list );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	/* Find the area detector record. */

	ad_record = mx_get_record( record_list, ad_name );

	if ( ad_record == NULL ) {
		mx_status = mx_error( MXE_NOT_FOUND, fname,
		    "Could not find the record '%s' in the MX database '%s'.",
			ad_name, database_filename );

		exit(mx_status.code);
	}

	/* Make sure some internal area detector parameters are initialized. */

	mx_status = mx_area_detector_get_image_format(ad_record, &format_type);

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_image_get_format_name_from_type( format_type,
							format_name,
							sizeof(format_name) );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_area_detector_get_framesize( ad_record,
						&row_framesize, &col_framesize);

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_area_detector_get_bytes_per_pixel( ad_record,
							&bytes_per_pixel );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	mx_status = mx_area_detector_get_bytes_per_frame( ad_record,
							&bytes_per_frame );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	fprintf( stderr, "Correcting %ldx%ld %s image file '%s'\n",
		row_framesize, col_framesize, format_name, original_filename );

	/* Load the correction files. */

	fprintf( stderr, "Loading correction files ... " );

	mx_status = mx_area_detector_load_correction_files( ad_record );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	fprintf( stderr, "Done\n" );

	/* Load the original image. */

	fprintf( stderr, "Loading image '%s' ... ", original_filename );

	mx_status = mx_area_detector_load_frame( ad_record,
						MXFT_AD_IMAGE_FRAME,
						original_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	fprintf( stderr, "Done\n" );

	/* Set the requested correction flags. */

	mx_status = mx_area_detector_set_correction_flags( ad_record,
							correction_flags );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	/* Correct the loaded frame. */

	fprintf( stderr, "Starting image correction ... " );

	mx_status = mx_area_detector_correct_frame( ad_record );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	fprintf( stderr, "Done\n" );

	/* Save the corrected frame. */

	fprintf( stderr, "Writing corrected frame to '%s' ... ",
					corrected_filename );

	mx_status = mx_area_detector_save_frame( ad_record,
						MXFT_AD_IMAGE_FRAME,
						corrected_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(mx_status.code);

	fprintf( stderr, "Done\n" );

	fprintf( stderr, "Offline correction complete.\n" );

	exit(0);
}

